From f7fdea8fe74e490798964b5daedfbc94b35f3baa Mon Sep 17 00:00:00 2001
From: Sebastian Pop <spop@amazon.com>
Date: Thu, 29 Jul 2021 21:28:49 +0000
Subject: [PATCH] [arm64] port scale1D_128to64 and scale2D_64to32

       scale1D_128to64	68.89x 	 12.06    	 830.58
        scale2D_64to32	86.83x 	 158.42   	 13756.12
---
 source/common/aarch64/asm-primitives.cpp |  6 ++++
 source/common/aarch64/pixel-util.S       | 44 ++++++++++++++++++++++++
 2 files changed, 50 insertions(+)

diff --git a/source/common/aarch64/asm-primitives.cpp b/source/common/aarch64/asm-primitives.cpp
index 126bbaf..3767290 100644
--- a/source/common/aarch64/asm-primitives.cpp
+++ b/source/common/aarch64/asm-primitives.cpp
@@ -295,6 +295,12 @@ void setupAssemblyPrimitives(EncoderPrimitives &p, int cpuMask)
         p.cu[BLOCK_16x16].count_nonzero   = PFX(count_nonzero_16_neon);
         p.cu[BLOCK_32x32].count_nonzero   = PFX(count_nonzero_32_neon);
 
+        //scale2D_64to32
+        p.scale2D_64to32  = PFX(scale2D_64to32_neon);
+
+        // scale1D_128to64
+        p.scale1D_128to64[NONALIGNED] = PFX(scale1D_128to64_neon);
+
         // copy_count
         p.cu[BLOCK_4x4].copy_cnt     = PFX(copy_cnt_4_neon);
         p.cu[BLOCK_8x8].copy_cnt     = PFX(copy_cnt_8_neon);
diff --git a/source/common/aarch64/pixel-util.S b/source/common/aarch64/pixel-util.S
index f901d9d..ea84e14 100644
--- a/source/common/aarch64/pixel-util.S
+++ b/source/common/aarch64/pixel-util.S
@@ -545,6 +545,50 @@ function x265_pixel_add_ps_8x16_neon
     ret
 endfunc
 
+// void scale1D_128to64(pixel *dst, const pixel *src)
+function x265_scale1D_128to64_neon
+.rept 2
+    ld2             {v0.16b, v1.16b}, [x1], #32
+    ld2             {v2.16b, v3.16b}, [x1], #32
+    ld2             {v4.16b, v5.16b}, [x1], #32
+    ld2             {v6.16b, v7.16b}, [x1], #32
+    urhadd          v0.16b, v0.16b, v1.16b
+    urhadd          v1.16b, v2.16b, v3.16b
+    urhadd          v2.16b, v4.16b, v5.16b
+    urhadd          v3.16b, v6.16b, v7.16b
+    st1             {v0.16b-v3.16b}, [x0], #64
+.endr
+    ret
+endfunc
+
+// void scale2D_64to32(pixel* dst, const pixel* src, intptr_t stride)
+function x265_scale2D_64to32_neon
+    mov             w12, #32
+.loop_scale2D:
+    sub             w12, w12, #1
+    ld1             {v0.16b-v3.16b}, [x1], x2
+    ld1             {v4.16b-v7.16b}, [x1], x2
+    uaddlp          v0.8h, v0.16b
+    uaddlp          v1.8h, v1.16b
+    uaddlp          v2.8h, v2.16b
+    uaddlp          v3.8h, v3.16b
+    uaddlp          v4.8h, v4.16b
+    uaddlp          v5.8h, v5.16b
+    uaddlp          v6.8h, v6.16b
+    uaddlp          v7.8h, v7.16b
+    add             v0.8h, v0.8h, v4.8h
+    add             v1.8h, v1.8h, v5.8h
+    add             v2.8h, v2.8h, v6.8h
+    add             v3.8h, v3.8h, v7.8h
+    uqrshrn         v0.8b, v0.8h, #2
+    uqrshrn2        v0.16b, v1.8h, #2
+    uqrshrn         v1.8b, v2.8h, #2
+    uqrshrn2        v1.16b, v3.8h, #2
+    st1             {v0.16b-v1.16b}, [x0], #32
+    cbnz            w12, .loop_scale2D
+    ret
+endfunc
+
 .macro x265_satd_4x8_8x4_end_neon
     add             v0.8h, v4.8h, v6.8h
     add             v1.8h, v5.8h, v7.8h
-- 
2.25.1

